@@ -1,20 +1,16 @@
 
 '''Tools for Bright Edge eServices developments & projects
 
-These tools was designed for the use in the Bright Edge eServices echo system.
-It defines methods and functions for general use purposes and standardization
-in the Bright Edge eServices echo system.
+Designed for the use in the Bright Edge eServices echo system. It defines
+methods and functions for general use purposes.
 
-The module define defaults for log levels, display on console, operating
-system names and date formats.
-
-The defaults are used in this module and across the Bright Edge eServices
-echo system.  The module basically has
+Archiver creates an archive of the key project files, print coloured messages
+to console with default parameters.
 
 To Do
 =====
-1.  Better example on the logging integration
-2.  Complete doctests for all methods & functions
+1. Better example on the logging integration.
+2. Complete doctests for all methods & functions.
 
 '''
 
@@ -26,7 +22,7 @@ from pathlib import Path
 import shutil
 import sys
 import tempfile
-from beetools.beemsg import header as msg_header
+from termcolor import colored
 
 _PROJ_DESC = __doc__.split('\n')[0]
 _PROJ_PATH = Path(__file__)
@@ -35,21 +31,19 @@ _PROJ_VERSION = '3.3.0'
 
 
 class Archiver:
-    '''Create an instance keep meta data about an application and backup
-    specified files to a zipfile during development.
-
-    Archiver assume the following project structure of the caling application:
+    '''Archiver creates an archive of the key project files to zip file. It assumes the
+    following project structure of the caling application:
 
     Module
     ======
-    projectname/ (a.k.a Project root folder)
+    projectname/ (a.k.a. Project root dir)
     │
     ├── Data/
     │
     ├── docs/
     │
-    ├── src/ (a.k.a Srouce folder)
-    │   └── modname (a.k.a Project folder
+    ├── src/ (a.k.a. Source dir)
+    │   └── modname (a.k.a. Project dir)
     │       ├── module1.py
     │       └── module2.py
     │
@@ -65,13 +59,13 @@ class Archiver:
 
     Package
     ======
-    projectname/ (a.k.a Project root folder)
+    projectname/ (a.k.a. Project root dir)
     │
     ├── Data/
     │
     ├── docs/
     │
-    ├── pkgname/ (a.k.a Project folder
+    ├── pkgname/ (a.k.a. Project dir)
     │   ├── __init__.py
     │   ├── file1.py
     │   └── file2.py
@@ -86,7 +80,7 @@ class Archiver:
     ├── requirements.txt
     └── setup.py
 
-    Library (not implimented)
+    Library (not implemented)
     =======
     projectname/
     │
@@ -125,14 +119,16 @@ class Archiver:
         p_app_ini_file_name=None,
         p_cls=True,
         p_logger = False,
-        p_archive_ext = None
+        p_arc_excl_dir = None,
+        p_arc_extern_dir = None,
+        p_arc_incl_ext = None
     ):
         '''Initialize the object
 
         Parameters
         ----------
         p_parent_log_name
-            Name of the parent logger i.e. calling application
+            Name of the parent logger, i.e., calling application
         p_app_ver
             Version of the calling application
         p_app_desc
@@ -141,10 +137,10 @@ class Archiver:
             Path to the application module
         p_app_ini_file_name
             Ini file name used by calling application for paramenters
-            Default is None
+            Default = None
         p_cls
             Clear the screen before start
-            Default is True
+            Default = True
 
         Returns
         -------
@@ -160,22 +156,27 @@ class Archiver:
         if p_logger:
             self.log_name = '{}.{}'.format(p_parent_log_name, _PROJ_NAME)
             self.logger = logging.getLogger(self.log_name)
-
+        else:
+            self.log_name = None
+            self.logger = None
         if not isinstance(p_app_pth, Path):
             self.app_pth = Path(p_app_pth)
         else:
             self.app_pth = p_app_pth
-        self.app_root_dir = self.find_app_root_dir()
+
         self.app_desc = p_app_desc
         self.app_ini_file_name = p_app_ini_file_name
         self.app_name = self.app_pth.stem
         self.app_ver = p_app_ver
-        self.archive_dir = None
-        if p_archive_ext is None:
-            self.archive_ext = ['ini', 'py']
+        self.app_root_dir = self.find_app_root_dir()
+        self.arc_dir = None
+        self.arc_excl_dir = _add_parm(['Archive', 'VersionArchive', 'build'], p_arc_excl_dir)
+        if p_arc_extern_dir:
+            self.arc_extern_dir = Path(p_arc_extern_dir)
         else:
-            self.archive_ext = p_archive_ext
-        self.archive_pth = None
+            self.arc_extern_dir = None
+        self.arc_incl_ext = _add_parm(['ini', 'py'], p_arc_incl_ext)
+        self.arc_pth = None
         self.cls = p_cls
         self.dur_hours = 0
         self.dur_min = 0
@@ -185,11 +186,13 @@ class Archiver:
         self.start_time = datetime.datetime.now()
         self.start_date_str = self.start_time.strftime('%y%m%d%H%M%S')
         self.version_archive = 'VersionArchive'
+
         self.make_archive()
+        self.make_archive_external()
         pass
 
     def is_dev_mode(self):
-        '''Determine if module is in production or not
+        '''Determine if it is a production module or not.
 
         Parameters
         ----------
@@ -225,20 +228,32 @@ class Archiver:
 
     def make_archive(self):
         if self.is_dev_mode() and self.app_root_dir:
-            self.archive_dir = self.app_root_dir / self.version_archive
-            if not self.archive_dir.is_dir():
-                self.archive_dir.mkdir()
-            self.archive_pth = self.archive_dir / '{} {} ({} Beta).zip'.format(
+            self.arc_dir = self.app_root_dir / self.version_archive
+            if not self.arc_dir.is_dir():
+                self.arc_dir.mkdir()
+            self.arc_pth = self.arc_dir / '{} {} ({} Beta).zip'.format(
                 self.app_name,
                 self.start_date_str,
                 self.app_ver
             )
-            with zipfile.ZipFile( self.archive_pth, 'w') as archive_zip:
-                for ext in self.archive_ext:
+            with zipfile.ZipFile(self.arc_pth, 'w') as archive_zip:
+                for ext in self.arc_incl_ext:
                     files = self.app_root_dir.glob('**/*.{}'.format(ext))
                     for file in files:
-                        if self.version_archive not in Path(file).parts:
+                        exclude_file = False
+                        for excl_dir in self.arc_excl_dir:
+                            if excl_dir in file.parts:
+                                exclude_file = True
+                        if not exclude_file:
                             archive_zip.write(file)
+                            pass
+            pass
+
+    def make_archive_external(self):
+        if self.is_dev_mode() and self.arc_extern_dir:
+            if not self.arc_extern_dir.exists():
+                self.arc_extern_dir.mkdir()
+            shutil.copy(self.arc_pth, self.arc_extern_dir)
 
     def print_footer(self):
         '''Print standard footers
@@ -281,7 +296,7 @@ class Archiver:
         return success
 
     def print_header(self, p_cls=True):
-        '''Initialize the start of the module,. make backup and print standard headers
+        '''Initialize the start of the module, make backup and print standard headers
 
         Parameters
         ----------
@@ -316,7 +331,165 @@ class Archiver:
         return success
 
 
-def do_examples(p_cls=True):
+# Message defaults
+BAR_LEN = 50
+MSG_LEN = 50
+CRASH_RETRY = 2
+
+
+def _add_parm(def_parm, new_parm):
+    if isinstance(new_parm, list):
+        def_parm += [x for x in new_parm if x not in def_parm]
+    elif isinstance(new_parm, str) and new_parm not in def_parm:
+        def_parm.append(new_parm)
+    return def_parm
+
+
+def msg_display(p_msg, p_len=MSG_LEN, p_color='white') -> str:
+    '''Return a text message in white on black.
+
+    Parameters
+    ----------
+    p_msg
+        The message
+    p_len
+        The fixed length of the message. Default is beetools.MSG_LEN
+    p_color
+        Color of text, always on black.
+            [ grey, red, green, yellow, blue, magenta, cyan, white ]
+
+    Returns
+    -------
+    str
+        Text in the specified color.
+
+    Examples
+    --------
+    >>> from beetools import msg.display
+    >>> msg.display( 'Display message' )
+    '\\x1b[37mDisplay message                               '
+
+    '''
+    msg = colored('{: <{len}}'.format(p_msg, len=p_len), p_color)
+    return msg[:p_len] + ' '
+
+
+def msg_error(p_msg) -> str:
+    '''Return an "error" text message in red on black
+
+    Parameters
+    ----------
+    p_msg
+        The message
+
+    Returns
+    -------
+    str
+        Text in red on black.
+
+    Examples
+    --------
+    >>> from beetools import msg.error
+    >>> msg.error( 'Error message' )
+    '\\x1b[31mError message\\x1b[0m'
+
+    '''
+    return colored('{}'.format(p_msg), 'red')
+
+
+def msg_header(p_msg) -> str:
+    '''Return a "header" text message in cyan on black
+
+    Parameters
+    ----------
+    p_msg
+        The message
+
+    Returns
+    -------
+    str
+        Text in red on black.
+
+    Examples
+    --------
+    >>> from beetools import msg.header
+    >>> msg.header( 'Header message' )
+    '\\x1b[36mHeader message\\x1b[0m'
+
+    '''
+    return colored('{}'.format(p_msg), 'cyan')
+
+
+def msg_info(p_msg) -> str:
+    '''Return an "information" text message in yellow on black
+
+    Parameters
+    ----------
+    p_msg
+        The message
+
+    Returns
+    -------
+    str
+        Text in red on black.
+
+    Examples
+    --------
+    >>> from beetools import msg.info
+    >>> msg.info( 'Info message' )
+    '\\x1b[33mInfo message\\x1b[0m'
+
+    '''
+    return colored('{}'.format(p_msg), 'yellow')
+
+
+def msg_milestone(p_msg) -> str:
+    '''Return a "milestone" text message in magenta on black
+
+    Parameters
+    ----------
+    p_msg
+        The message
+
+    Returns
+    -------
+    str
+        Text in red on black.
+
+    Examples
+    --------
+    >>> from beetools import msg.milestone
+    >>> msg.milestone( 'Milestone message' )
+    '\\x1b[35mMilestone message\\x1b[0m'
+
+    '''
+    return colored('{}'.format(p_msg), 'magenta')
+
+
+def msg_ok(p_msg) -> str:
+    '''Return an "OK" text message in green on black
+
+    Parameters
+    ----------
+    p_msg
+        The message
+
+    Returns
+    -------
+    str
+        Text in red on black.
+
+    Examples
+    --------
+    >>> from beetools import msg.ok
+    >>> msg.ok( 'OK message' )
+    '\\x1b[32mOK message\\x1b[0m'
+
+    '''
+    return colored('{}'.format(p_msg), 'green')
+
+
+def example_archiver(p_cls=True):
     '''Example to illustrate usage
 
     Parameters
@@ -328,7 +501,7 @@ def do_examples(p_cls=True):
     Returns
     -------
     bool
-        Successful execution [ b_tls.archive_pth | False ]
+        Successful execution [ b_tls.arc_pth | False ]
 
     Examples
     --------
@@ -344,13 +517,51 @@ def do_examples(p_cls=True):
         app_dir.mkdir(parents = True)
         app_pth = app_dir / Path(app_name.lower()).with_suffix('.py')
         app_pth.touch()
-        t_archiver = Archiver(app_name, app_version, app_desc, app_pth)
+        arc_extern_dir = Path(temp_dir, 'external')
+        # arc_extern_dir.mkdir(parents = True)
+        t_archiver = Archiver(app_name, app_version, app_desc, app_pth, p_arc_extern_dir = arc_extern_dir)
     t_archiver.print_header(p_cls = p_cls)
     t_archiver.print_footer()
     # working_dir.rmdir()
     return success
 
 
+def example_messaging():
+    '''Standard example to illustrate standard use.
+
+    Parameters
+    ----------
+
+    Returns
+    -------
+    bool
+        Successful execution [ b_tls.archive_path | False ]
+
+    Examples
+    --------
+
+    '''
+    success = True
+    print(
+        msg_display(
+            'This message print in blue and cut at {} character because it is too long!'.format(
+                MSG_LEN),
+            p_color = 'blue'
+        )
+    )
+    print(msg_ok('This message is an OK message'))
+    print(msg_info('This is an info message'))
+    print(msg_milestone('This is a milestone message'))
+    print(msg_error('This is a warning message'))
+    return success
+
+
+def do_examples(p_cls = True):
+    success = True
+    success = example_archiver(p_cls = p_cls) and success
+    success = example_messaging() and success
+    return success
+
+
 if __name__ == '__main__':
-    # do_examples(sys.argv[0])
     do_examples()
