@@ -384,23 +384,47 @@ def rm_tree(p_pth, p_crash = True):
 
     '''
     success = False
-    tree = list(p_pth.glob('**/*'))
-    not_empty = []
-    while len(tree) > 0:
-        for src in tree:
-            if src.is_file():
+    # tree = list(p_pth.glob('**/*'))
+    # not_empty = []
+    locked_file_lst = []
+    crash_retry_cntr = 0
+    # success = False
+    while crash_retry_cntr < CRASH_RETRY and not success:
+        success = True
+        for root, dirs, files in os.walk(p_pth, topdown=False):
+            for file in files:
                 try:
-                    src.unlink()
-                except PermissionError:
-                    rm_temp_locked_file(src)
-            elif src.is_dir():
+                    file_pth = os.path.join(root, file)
+                    os.remove(file_pth)
+                    success = True and success
+                except PermissionError as err:
+                    # locked_file_lst.append(file_pth)
+                    success = False
+            for dir in dirs:
                 try:
-                    src.rmdir()
-                except OSError:
-                    if src.exists():
-                        not_empty.append(src)
-        tree = not_empty.copy()
-        not_empty = []
+                    dir_pth = os.path.join(root, dir)
+                    os.rmdir(dir_pth)
+                    success = True and success
+                except OSError as err:
+                    success = False
+        crash_retry_cntr += 1
+
+    # while len(tree) > 0:
+    #     for src in tree:
+    #         if src.is_file():
+    #             try:
+    #                 src.unlink()
+    #             except PermissionError as err:
+    #                 locked_file_lst.append(src)
+    #                 rm_temp_locked_file(src)
+    #         elif src.is_dir():
+    #             try:
+    #                 src.rmdir()
+    #             except OSError as err:
+    #                 if src.exists():
+    #                     not_empty.append(src)
+    #     tree = not_empty.copy()
+    #     not_empty = []
     if p_pth.exists():
         try:
             p_pth.rmdir()
